clear
clc
startepoch = '8a6a20f3'; %preset the first epoch recorded. Eve started recording earlier than bob
endepoch = '8a6a2383'; %preset the last epoch recorded. Eve stopped recording later than bob
startepochnum = hex2dec(startepoch);
endepochnum = hex2dec(endepoch);
%source files
t1source = '../data-raw/eve-raw-events/'; %Eve's raw recorded data, her clicks
t2source= '../data-raw/alice-receivefiles/'; %time and basis information transmitted from low count rate side (bob)
t3bobsource = '../data-raw/bob-siftedkey/'; %Bob's sifted key
t4source = '../data-raw/bob-receivefiles/'; %index of sift information reutrned by the other side (alice)

if exist(t2source, 'dir')==0 || exist(t3bobsource, 'dir')==0 || exist(t4source, 'dir')==0
    disp('source files not exist')
end

%newly generated files
t3bobuncompressdir = '../data-produced-by-scripts/bob-siftedkey-ascii/'; %will store Bob's sifted key in ascii files
t3eveuncompressdir = '../data-produced-by-scripts/eve-siftedkey-ascii/'; %will store Eve's sifted key in ascii files
t3everawkey = '../data-produced-by-scripts/eve-siftedkey/'; %store Eve's sifted key in t3 file
log='proclog.txt';

%prepare the directories
func_createdir(t3bobuncompressdir); 
func_createdir(t3eveuncompressdir);
func_createdir(t3everawkey);

disp('Finding time difference...')
diffsec = func_findtimedifference(t1source, t2source, startepoch); % find time difference between eve and bob, the unit is ns
diff=8*diffsec; %convert the unit to 0.125ns, which will be used later
disp(['Time difference:' num2str(diffsec) 'ns'])

disp('Generating Eve sifted key in expanded version...')
for (epochnumber = startepochnum:endepochnum) 
    
    %show the generating progress in X%
    thres=floor((endepochnum-startepochnum)/10);
    if mod((epochnumber-startepochnum), thres)==0
        disp([num2str(10*(epochnumber-startepochnum)/thres), '%']) %display the progress
    end
    
    readepoch = lower(dec2hex(epochnumber)); %the current reading epoch
    t1tab1=func_uncompresst1(t1source,readepoch); %call the function: uncompress Eve's data in a nx2 matrix, in which one column is timing and the other column is the bit value of each click
    t2tab=func_uncompresst2(t2source,readepoch);  
    t4tab=func_uncompresst4(t4source,readepoch);  
    t1tab2=func_uncompresst1(t1source,lower(dec2hex(hex2dec(readepoch)+1))); %get Eve's clicks from the next epoch
    t1tab=[t1tab1',t1tab2']'; %splice Eve's data of the current epoch and the next epoch
    t1=t1tab(:,1); %get the timing
    t2=t2tab(:,1); %get the timing

    off1=fix(t1(1)/(2^32)) * (2^32);
    t1=t1-off1; %make all timings refer to the beginning of the epoch

    for i=2:length(t2) 
        t2(i)=t2(i)+t2(i-1);
    end
    
    t2=t2+diff;
    
    skelen=length(t4tab);
    sk=zeros(skelen,1); % will store eve's key in raw value: 1, 2, 4, 8
    ske=zeros(skelen,1); % will store eve's key in useful value: 0,1
    %skecheck=zeros(skelen,2); % first column is the timing of eve's key, and second column is the timing of bob's corresponding key 
    t4=t4tab;
    
    for i=2:skelen
        t4(i)=t4(i)+t4(i-1);
    end
    t4=t4+1;
    ii=1;
    for i=1:skelen
        t2num=t4(i);
        compa=t2(t2num); % the timing of bob's key as a reference to sift eve's key
        while (t1(ii)<compa)
            ii=ii+1;
        end
        if t1(ii)-compa<compa-t1(ii-1)
            sk(i)=t1tab(ii,2);
          %  skecheck(i,1)=t1tab(ii,1);
        else
            sk(i)=t1tab(ii-1,2);        
            %skecheck(i,1)=t1(ii-1);
            ii=ii-1; 
        end       
      %  skecheck(i,2)=t2tab(t2num,1);
    end
    
    for i=1:skelen
        if sk(i)==1 || sk(i)==2
            ske(i)=0;
        else            
            ske(i)=1;
        end    
    end
    dlmwrite(strcat(t3eveuncompressdir,readepoch),ske,'precision',1);
    %dlmwrite(strcat(skecheckdir,readepoch),skecheck,'precision',15);
end
disp(['Eve sifted key generated in directory ' t3eveuncompressdir])

disp(['expanding t3 files (Bob sifted key) to ' t3bobuncompressdir])
disp('...')
for (epochnumber = startepochnum:endepochnum)
    readepoch = lower(dec2hex(epochnumber)); %make sure all lowercase
    t3tab = func_uncompresst3(t3bobsource,readepoch);
    dlmwrite([t3bobuncompressdir,readepoch],t3tab,'precision', 1);
end

disp('Comparing Eve and Bob sifted key...')
totaleve=0;
totalbob=0;
error=0;
for (epochnumber = startepochnum:endepochnum)
    readepoch = lower(dec2hex(epochnumber));
    ske=load(strcat(t3eveuncompressdir,readepoch));
    skb=load(strcat(t3bobuncompressdir,readepoch));
    totaleve=totaleve+length(ske);
    totalbob=totalbob+length(skb);
    for i=1:length(ske)
        if ske(i)~=skb(i)
            error=error+1;
        end
    end
end
errorpercentage=100*error/totalbob;
disp(['Total bits in sifted key of Bob: ' num2str(totalbob)]);
disp(['Total bits in sifted key of Eve: ' num2str(totaleve)]);
disp(['Total errors: ' num2str(error)]);
disp(['Percentage of errors: ' num2str(errorpercentage) '%']);

disp(['Storing Eve sifted key in t3 files ' t3everawkey])
for (epochnumber = startepochnum:endepochnum)
    readepoch = lower(dec2hex(epochnumber));
    ske=load(strcat(t3eveuncompressdir,readepoch));
    len=length(ske);
    
    data=zeros(1,ceil(len/32));
    lendata=length(data);
    data=[259,epochnumber,len,1,data];
    ske1=padarray(ske,lendata*32-len,'post');
    for i=1:lendata
        data(i+4)=bin2dec(char(ske1(32*i-31:32*i)'+48));
    end
    
    fid = fopen(strcat(t3everawkey, readepoch), 'w');
    fwrite(fid, data, 'uint32');
    fclose(fid);    
end
disp('Finished')

fid = fopen(log, 'w');
fprintf(fid, [' Start epoch: ' startepoch ' \n']);
fprintf(fid, [' End epoch: ' endepoch ' \n']);
fprintf(fid, [' Time difference of Bob and Eve: ' num2str(diffsec) 'ns \n']);
fprintf(fid, [' Total bits in sfited key of Bob: ' num2str(totalbob) ' \n']);
fprintf(fid, [' Total bits in sifted key of Eve: ' num2str(totaleve) ' \n']);
fprintf(fid, [' Total errors: ' num2str(error) ' \n']);
fprintf(fid, [' Percentage of errors: ' num2str(errorpercentage) '% \n']);
fclose(fid); 
